--[[
Callback events  

TEAM_UPDATED 
	args: teamIndex or nil
		nil indicates all teams should be refreshed
		teamindex indicates which team should refresh
		
TEAM_DELETED
	args: teamIndex
		indicates which team was deleted
		
TEAM_CREATED
	args: teamIndex
		indicates the newly created teams index

]]


local PetBattleTeams = LibStub("AceAddon-3.0"):GetAddon("PetBattleTeams")
TeamManager = PetBattleTeams:NewModule("TeamManager")
local PETS_PER_TEAM = 3
local START = 1
local FINISHED = 5

local function OnUpdate()
	local self = TeamManager
	local selected = self:GetSelected()
	local petSlot = self.petSlot
	
	if petSlot <= PETS_PER_TEAM  then
		
		local petID, abilities = self:GetPetInfo(selected,petSlot)
		local isValidPet = C_PetJournal.GetPetInfoByPetID(petID)
		local currentPetID, ability1, ability2, ability3, locked = C_PetJournal.GetPetLoadOutInfo(petSlot)
		
		if not isValidPet then
			C_PetJournal.SetPetLoadOutInfo(petSlot,0)
			if C_PetJournal.GetPetLoadOutInfo(petSlot) == 0 then
				self.petSlot = petSlot +1
			end
			return
		end
		
		if locked then 
			C_PetJournal.SetPetLoadOutInfo(petSlot,0)
			if C_PetJournal.GetPetLoadOutInfo(petSlot) == 0 then
				self.petSlot = petSlot +1
				return
			end
		end
		
		if petID ~= currentPetID then 
			C_PetJournal.SetPetLoadOutInfo(petSlot,petID) 
			return
		else
			if ability1 ~= abilities[1] then 
				C_PetJournal.SetAbility(petSlot, 1, abilities[1])
				return
			elseif ability2 ~= abilities[2] then
				C_PetJournal.SetAbility(petSlot, 2, abilities[2])
				return
			elseif ability3 ~= abilities[3] then
				C_PetJournal.SetAbility(petSlot, 3, abilities[3])
				return
			end
		end
		
		if petID == currentPetID and ability1 == abilities[1] and ability2 == abilities[2] and ability3 == abilities[3] then
			self.petSlot = petSlot +1
			return
		end
	else
		PetJournal_UpdatePetLoadOut()
		self.frame:SetScript("OnUpdate",nil)
		self.petSlot = FINISHED
	end 
end

function TeamManager:GetPetInfo(teamIndex,petIndex)
	assert(type(teamIndex) == "number" and type(petIndex) == "number")
	if self.teams[teamIndex] and self.teams[teamIndex][petIndex] then
		local pet = self.teams[teamIndex][petIndex]
		if pet then
			local abilities = pet.abilities
			return pet.petID, abilities
		end
	end
	return 0
end

function TeamManager:GetNumPets(teamIndex)
	assert(type(teamIndex) == "number")
	if self.teams[teamIndex] then 
		return #self.teams[teamIndex]
	end
	return 0
end

function TeamManager:GetNumTeams()
	return #self.teams
end

function TeamManager:GetTeamName(teamIndex)
	if type(teamIndex) ~= "number" then return "Unknown Team" end
	
	local name = "Team: "..tostring(teamIndex)
	local customName
	
	if self.teams[teamIndex] and self.teams[teamIndex].name then
		customName = self.teams[teamIndex].name
	end
	
	local displayName = customName and customName or name
	
	return displayName, name, customName
end

function TeamManager:SetTeamName(teamIndex,name)
	assert(type(teamIndex) == "number"  and  (type(name) == "string" or name == nil))
	if  self.teams[teamIndex] then
		self.teams[teamIndex].name = name
	end
	self.callbacks:Fire("TEAM_UPDATED",teamIndex)
end

function TeamManager:MoveTeam(teamIndexSource,teamIndexDesintation)
	assert(type(teamIndexSource) == "number" and type(teamIndexDesintation) == "number")
	if self.teams[teamIndexSource] and self.teams[teamIndexDesintation] then
		
		local selected = self:GetSelected()
		local distance = math.abs(teamIndexSource - teamIndexDesintation)
		
		if true then --distance > 1 then
			local tempTeam = self.teams[teamIndexSource] 
			table.remove(self.teams,teamIndexSource)
			
			 --adjust for the fact that we delete first to make sure their is consistant behavior for the user
			if teamIndexSource < teamIndexDesintation then
				teamIndexDesintation = teamIndexDesintation - 1
			end
			
			table.insert(self.teams,teamIndexDesintation ,tempTeam )
			
			--if the swap happens in such as way as to re-order the teams pushing the selected team to another team number
			--then update the selected team to the predicted new location of the old selected team
			
			--if source && desintation < selected or source && desintation > selected do nothing
			--if destination == selected and source < selected then post move do nothing
			
			--if source < selected and destination > selected then post move, selected is -1
			--if source == selected then post move selected is desintation - 1
			
			--if destination == selected and source > selected then post move selected +1
			--if source > selected and destination < selected then post move, selected is +1
			
			local adjust = teamIndexSource <  teamIndexDesintation and -1 or 1
			
			if teamIndexSource == selected then
				self:SetSelected(teamIndexDesintation)
			elseif teamIndexDesintation == selected   then
				self:SetSelected(teamIndexDesintation + adjust)
			elseif (teamIndexSource < selected and teamIndexDesintation > selected) or (teamIndexSource > selected and teamIndexDesintation < selected) then
				self:SetSelected(selected + adjust)
			end
			
			self.callbacks:Fire("TEAM_UPDATED")
		--[[else 
			local tempTeam = self.teams[teamIndexSource] 
			self.teams[teamIndexSource]  = self.teams[teamIndexDesintation]
			self.teams[teamIndexDesintation] = tempTeam
			
			--swap selection
			if selected == teamIndexSource then
				self:SetSelected(teamIndexDesintation)
			elseif selected == teamIndexDesintation then
				self:SetSelected(teamIndexSource)
			end

			self.callbacks:Fire("TEAM_UPDATED")]]
		end
	end
end

function TeamManager:CreateTeam()
	local numTeams, maxTeams = self:GetNumTeams()
	
	local team = {}
	team.name = nil;
	for i = 1,PETS_PER_TEAM do
		pet = {}
		pet.abilities = {}
		pet.petID, pet.abilities[1], pet.abilities[2], pet.abilities[3] = C_PetJournal.GetPetLoadOutInfo(i)
		table.insert(team,pet)
	end
	table.insert(self.teams,team)
	self.callbacks:Fire("TEAM_CREATED",self:GetNumTeams())
	self.callbacks:Fire("TEAM_UPDATED",self:GetNumTeams())
end

function TeamManager:TeamExists(teamIndex)
	assert(type(teamIndex) == "number")
	if self.teams[teamIndex] then return true end
	return false
end

function TeamManager:TeamContainsPet(teamIndex,petID)
	assert(type(teamIndex) == "number")
	
	if not self.teams[teamIndex] or not petID or petID <= 0 then return false end
	local numPets = TeamManager:GetNumPets(teamIndex)
	for i=1,numPets do
		local teamPetID = TeamManager:GetPetInfo(teamIndex,i)
		if teamPetID == petID then
			return true,i
		end
	end
	return false
end

function TeamManager:DeleteTeam(teamIndex)
	assert(type(teamIndex) == "number")
	
	
	local numTeams = self:GetNumTeams()
	local selectedTeam = self:GetSelected()
	
	if numTeams > 1 then
		table.remove(self.teams,teamIndex)
		
		if teamIndex == numTeams then
			self:SetSelected(teamIndex-1)
		elseif teamIndex == selectedTeam then
			self:SetSelected(teamIndex)
		end
		
		self.callbacks:Fire("TEAM_DELETED",teamIndex)
	end
end

function TeamManager:IsWorking()
	return self.petSlot ~= FINISHED
end

function TeamManager.UpdateCurrentTeam()
	local selected = TeamManager:GetSelected()
	if not TeamManager:IsWorking() and not TeamManager:IsTeamLocked(selected)  then
		local team = {}
		for i = 1,PETS_PER_TEAM do
			pet = {}
			pet.abilities = {}
			pet.petID, pet.abilities[1], pet.abilities[2], pet.abilities[3] = C_PetJournal.GetPetLoadOutInfo(i)
			table.insert(team,pet)
		end
		TeamManager.teams[selected] = team
		TeamManager.callbacks:Fire("TEAM_UPDATED",selected)
	end
end

function TeamManager:GetSelected()
	return self.db.global.selected 
end

function TeamManager:IsSelected(teamIndex)
	assert(type(teamIndex) == "number")
	return self.db.global.selected  == teamIndex
end 


function TeamManager:SetSelected(teamIndex)
	assert(type(teamIndex) == "number" )
	local selected = self:GetSelected()
	if teamIndex <= 0 or teamIndex > self:GetNumTeams() or self:IsTeamLocked(selected) then return end
	self.db.global.selected = teamIndex
	self.callbacks:Fire("SELECTED_TEAM_CHANGED",teamIndex)
	self:ApplyTeam(teamIndex)
end

function TeamManager:IsTeamLocked(teamIndex)
	assert(type(teamIndex) == "number")
	if teamIndex == self:GetSelected() and (C_PetBattles.GetPVPMatchmakingInfo() or C_PetBattles.IsInBattle())  then return true end
	return false
end

function TeamManager:UpdateTeamNewPet(petID,teamIndex,petIndex,sourceTeamIndex,sourcePetIndex)
	assert(type(petID) == "number" and petID > 0)
	assert(type(petIndex) == "number" and petIndex <= PETS_PER_TEAM and petIndex > 0)
	assert(type(teamIndex) == "number")

	if self:IsTeamLocked(teamIndex)  then return end
	if self:TeamContainsPet(teamIndex,petID) then return end
	local speciesID, customName, level, xp, maxXp, displayID, name, icon, petType, creatureID, sourceText, description, isWild, canBattle, tradable = C_PetJournal.GetPetInfoByPetID(petID)
	
	if speciesID and canBattle then
		local pet = {}
		pet.petID = petID
		pet.abilities = {}
		if sourceTeamIndex and sourcePetIndex and  sourcePetIndex < PETS_PER_TEAM then
			local _, sourceAbilities = self:GetPetInfo(sourceTeamIndex,sourcePetIndex)
			
			for i=1,3 do
				pet.abilities[i] = sourceAbilities[i]
			end
			
		else
			local abilities = {}
			C_PetJournal.GetPetAbilityList(speciesID, abilities, {})
			for i=1,3 do
				pet.abilities[i] = abilities[i]
			end
		end
			
		self.teams[teamIndex][petIndex] = pet;
		if self:IsSelected(teamIndex) then
			self:ApplyTeam(teamIndex)
		end
		self.callbacks:Fire("TEAM_UPDATED",teamIndex)
	end
end

function TeamManager:UpdateTeamSwapPets(destinationTeam,destinationPetIndex,sourceTeam,sourcePetIndex)
	assert(type(destinationTeam) == "number")
	assert(type(destinationPetIndex) == "number" and destinationPetIndex <= PETS_PER_TEAM and destinationPetIndex > 0)
	assert(type(sourceTeam) == "number")
	assert(type(sourcePetIndex) == "number" and sourcePetIndex <= PETS_PER_TEAM and sourcePetIndex > 0)
	
	local selected = self:GetSelected()
	if self:IsTeamLocked(destinationTeam) or self:IsTeamLocked(sourceTeam) then return end
	
	local petA = self.teams[destinationTeam][destinationPetIndex]
	local petB = self.teams[sourceTeam][sourcePetIndex]
	
	if destinationTeam ~=  sourceTeam and (self:TeamContainsPet(destinationTeam,petB.petID) or self:TeamContainsPet(sourceTeam,petA.petID)) then return end
	
	self.teams[destinationTeam][destinationPetIndex] = petB
	self.teams[sourceTeam][sourcePetIndex] = petA
	
	if selected == sourceTeam or selected == destinationTeam then 
		self:ApplyTeam(selected)
	end
	
	
	self.callbacks:Fire("TEAM_UPDATED",destinationTeam)
	self.callbacks:Fire("TEAM_UPDATED",sourceTeam)
end

function TeamManager:OnInitialize()
	self.callbacks = LibStub("CallbackHandler-1.0"):New(self)
	self.frame = CreateFrame("frame")
	self.petSlot = FINISHED
	
	local defaults = {
		global = {
			teams = {},
			selected = 0,
			hasImported = false,
			userLocked,
		}
	}

	-- Assuming the .toc says ## SavedVariables: MyAddonDB
	local db = LibStub("AceDB-3.0"):New("PetBattleTeamsDB", {} , true)
	local name = self:GetName()
	self.db = db:RegisterNamespace(name, defaults)
	
	
	
	self.teams = self.db.global.teams 
	if #self.teams == 0 then
		self:CreateTeam()
		self:SetSelected(1)
	end
	
	if self.db.global.hasImported == false then
		self:ImportTeams(PetBattleTeamsSettings.teams)
	end
	
	if #self.teams ~= 0 and self:GetSelected() == 0 then
		self:SetSelected(1)
	end
	 
	hooksecurefunc(C_PetJournal, "SetPetLoadOutInfo", TeamManager.UpdateCurrentTeam)
	hooksecurefunc(C_PetJournal, "SetAbility", TeamManager.UpdateCurrentTeam)
end


function TeamManager:ApplyTeam(teamIndex)
	assert(type(teamIndex) == "number")
	self.petSlot = START
	self.frame:SetScript("OnUpdate",OnUpdate)
end

--/run TeamManager:ImportTeams(PetBattleTeamsSettings.teams)
function TeamManager:ImportTeams(teams)
	assert(type(teams) == "table")
	
	local start = self:GetNumTeams() +1
	for i=1,#teams do
		--teams[i].name = "imported"
		table.insert(self.teams,teams[i])
	end
	self.db.global.hasImported = true
	self:SetSelected(1)
	self.callbacks:Fire("TEAM_UPDATED")
	print("PetBattle Teams: Imported",#teams,"Teams.")
end










